@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2011, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,9 +25,13 @@ import difflib
 
 from mysql.utilities.exception import UtilError, UtilDBError
 from mysql.utilities.common.format import print_list
-from mysql.utilities.common.pattern_matching import REGEXP_QUALIFIED_OBJ_NAME
+from mysql.utilities.common.pattern_matching import (
+    parse_object_name,
+    REGEXP_QUALIFIED_OBJ_NAME,
+    REGEXP_QUALIFIED_OBJ_NAME_AQ)
 from mysql.utilities.common.database import Database
 from mysql.utilities.common.lock import Lock
+from mysql.utilities.common.options import PARSE_ERR_OBJ_NAME_FORMAT
 from mysql.utilities.common.server import connect_servers
 from mysql.utilities.common.table import Table
 from mysql.utilities.common.sql_transform import (is_quoted_with_backticks,
@@ -51,6 +55,10 @@ _COMPARE_TABLE_DROP = """
 # the key length could be calculated by the number of rows.
 DEFAULT_SPAN_KEY_SIZE = 8
 
+# Max allowed size for the span_key. Must be smaller or equal than the size of
+# the key hash because it is a substring of it. Note: 32 = binary(16).
+MAX_SPAN_KEY_SIZE = 32
+
 # Note: Use a composed index (span, pk_hash) instead of only for column "span"
 # due to the "ORDER BY pk_hash" in the _COMPARE_DIFF query.
 _COMPARE_TABLE = """
@@ -103,7 +111,7 @@ _WARNING_INDEX_NOT_USABLE = ("# Warning: Specified index {idx} for table {tb}"
                              " cannot be used. It contains at least one "
                              "column that accepts null values.")
 
-_RE_EMPTY_ALTER_TABLE = "^ALTER TABLE {0};$".format(REGEXP_QUALIFIED_OBJ_NAME)
+_RE_EMPTY_ALTER_TABLE = "^ALTER TABLE {0};$"
 
 _RE_DASHES_DIG = re.compile(r"^\-{3}\s\d+")
 
@@ -150,10 +158,15 @@ def get_create_object(server, object_name, options, object_type):
     verbosity = options.get("verbosity", 0)
     quiet = options.get("quiet", False)
 
-    m_obj = re.match(REGEXP_QUALIFIED_OBJ_NAME, object_name)
-    db_name, obj_name = m_obj.groups()
+    # Get the sql_mode set on server
+    sql_mode = server.select_variable("SQL_MODE")
+
+    db_name, obj_name = parse_object_name(object_name, sql_mode)
     obj = [db_name]
 
+    if db_name is None:
+        raise UtilError(PARSE_ERR_OBJ_NAME_FORMAT.format(
+            obj_name=object_name, option=object_type.lower()))
     db = Database(server, obj[0], options)
 
     # Error if database does not exist
@@ -170,11 +183,11 @@ def get_create_object(server, object_name, options, object_type):
     if verbosity > 0 and not quiet:
         if obj_name:
             print("\n# Definition for object {0}.{1}:"
-                  "".format(remove_backtick_quoting(db_name),
-                            remove_backtick_quoting(obj_name)))
+                  "".format(remove_backtick_quoting(db_name, sql_mode),
+                            remove_backtick_quoting(obj_name, sql_mode)))
         else:
             print("\n# Definition for object {0}:"
-                  "".format(remove_backtick_quoting(db_name)))
+                  "".format(remove_backtick_quoting(db_name, sql_mode)))
         print create_stmt
 
     return create_stmt
@@ -235,7 +248,9 @@ def server_connect(server1_val, server2_val, object1, object2, options):
     if server2 is None:
         server2 = server1
 
-    if server1 == server2 and object1 == object2:
+    # Check if the specified objects and servers are the same.
+    if object1 == object2 and server1.port == server2.port and \
+       server1.is_alias(server2.host):
         raise UtilError("Comparing the same object on the same server.")
 
     return (server1, server2)
@@ -382,10 +397,11 @@ def _get_transform(server1, server2, object1, object2, options,
     """
 
     try:
-        m_obj1 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object1)
-        db1, name1 = m_obj1.groups()
-        m_obj2 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object2)
-        db2, name2 = m_obj2.groups()
+        db1, name1 = parse_object_name(object1,
+                                       server1.select_variable("SQL_MODE"))
+
+        db2, name2 = parse_object_name(object2,
+                                       server2.select_variable("SQL_MODE"))
     except:
         raise UtilError("Invalid object name arguments for _get_transform"
                         "(): %s, %s." % (object1, object2))
@@ -409,7 +425,7 @@ def _get_transform(server1, server2, object1, object2, options,
     # Get the transformation based on direction.
     transform_str = []
     xform = SQLTransformer(db_1, db_2, obj1[0], obj2[0], object_type,
-                           options.get('verbosity', 0))
+                           options.get('verbosity', 0), options)
 
     differences = xform.transform_definition()
     if differences and len(differences) > 0:
@@ -439,10 +455,11 @@ def _check_tables_structure(server1, server2, object1, object2, options,
     the third is a boolean indicating if the partition options are the same.
     """
     try:
-        m_obj1 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object1)
-        db1, name1 = m_obj1.groups()
-        m_obj2 = re.match(REGEXP_QUALIFIED_OBJ_NAME, object2)
-        db2, name2 = m_obj2.groups()
+        db1, name1 = parse_object_name(object1,
+                                       server1.select_variable("SQL_MODE"))
+
+        db2, name2 = parse_object_name(object2,
+                                       server2.select_variable("SQL_MODE"))
     except:
         raise UtilError("Invalid object name arguments for diff_objects(): "
                         "{0}, {1}.".format(object1, object2))
@@ -645,8 +662,12 @@ def diff_objects(server1, server2, object1, object2, options, object_type):
     # Check if ALTER TABLE statement have changes. If not, it is probably
     # because there are differences but they have no influence on the create
     # table, such as different order on indexes.
-    if (diff_list and same_tbl_def and same_part_def
-       and re.match(_RE_EMPTY_ALTER_TABLE, diff_list[1])):
+    if "ANSI_QUOTES" in server1.select_variable("SQL_MODE"):
+        regex_pattern = REGEXP_QUALIFIED_OBJ_NAME_AQ
+    else:
+        regex_pattern = _RE_EMPTY_ALTER_TABLE.format(REGEXP_QUALIFIED_OBJ_NAME)
+    if diff_list and same_tbl_def and same_part_def and \
+       re.match(regex_pattern, diff_list[1]):
         print("[PASS]")
         return None
 
@@ -675,8 +696,9 @@ def diff_objects(server1, server2, object1, object2, options, object_type):
         for line in diff_list:
             print line
 
-        print("# WARNING: Could not generate changes for {0}. No changes "
-              "required or not supported difference.")
+        print("# WARNING: Could not generate SQL statements for differences "
+              "between {0} and {1}. No changes required or not supported "
+              "difference.".format(object1, object2))
 
         return diff_list
 
@@ -716,18 +738,27 @@ def _drop_compare_object(server, db_name, tbl_name):
     tbl_name[in]           table name
     """
     # Quote compare table appropriately with backticks
-    q_db_name = db_name if is_quoted_with_backticks(db_name) \
-        else quote_with_backticks(db_name)
-    if is_quoted_with_backticks(tbl_name):
-        q_tbl_name = remove_backtick_quoting(tbl_name)
+    sql_mode = server.select_variable("SQL_MODE")
+    q_db_name = db_name if is_quoted_with_backticks(db_name, sql_mode) \
+        else quote_with_backticks(db_name, sql_mode)
+    if is_quoted_with_backticks(tbl_name, sql_mode):
+        q_tbl_name = remove_backtick_quoting(tbl_name, sql_mode)
     else:
         q_tbl_name = tbl_name
     q_tbl_name = quote_with_backticks(
-        _COMPARE_TABLE_NAME.format(tbl=q_tbl_name))
+        _COMPARE_TABLE_NAME.format(tbl=q_tbl_name), sql_mode)
 
     try:
+        # set autocommit=1 if it is 0, because CREATE TEMPORARY TABLE and
+        # DROP TEMPORARY TABLE can be executed in a non-transactional context
+        # only, and require that AUTOCOMMIT = 1.
+        toggle_server = not server.autocommit_set()
+        if toggle_server:
+            server.toggle_autocommit(enable=True)
         server.exec_query(_COMPARE_TABLE_DROP.format(db=q_db_name,
                                                      compare_tbl=q_tbl_name))
+        if toggle_server:
+            server.toggle_autocommit(enable=False)
     except:
         pass
 
@@ -752,22 +783,24 @@ def _get_compare_objects(index_cols, table1,
     table = None
 
     # build primary key col definition
-    index_str = ''.join("{0}, ".format(quote_with_backticks(col[0]))
+    index_str = ''.join("{0}, ".format(quote_with_backticks(col[0],
+                                                            table1.sql_mode))
                         for col in index_cols)
     index_defn = ''.join("{0} {1}, ".
-                         format(quote_with_backticks(col[0]), col[1])
+                         format(quote_with_backticks(col[0], table1.sql_mode),
+                                col[1])
                          for col in index_cols)
     if index_defn == "":
         raise UtilError("Cannot generate index definition")
     else:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name), table1.sql_mode)
 
         table = _COMPARE_TABLE.format(db=table1.q_db_name,
                                       compare_tbl=q_tbl_name,
                                       pkdef=index_defn,
-                                      span_key_size=span_key_size/2)
+                                      span_key_size=span_key_size / 2)
 
     return (table, index_str)
 
@@ -892,16 +925,36 @@ def _setup_compare(table1, table2, span_key_size, use_indexes=None):
     # Build the primary key hash if needed
     tbl1_table, pri_idx1 = _get_compare_objects(table1_idx, table1,
                                                 span_key_size)
-    tbl2_table, pri_idx2 = _get_compare_objects(table1_idx, table2,
+    tbl2_table, pri_idx2 = _get_compare_objects(table2_idx, table2,
                                                 span_key_size)
 
     if tbl1_table is None or tbl2_table is None:
         raise UtilError("Cannot create compare table.")
 
     # Create the compare tables
+
+    # set autocommit=1 if it is 0, because CREATE TEMPORARY TABLE and DROP
+    # TEMPORARY TABLE can be executed in a non-transactional context only, and
+    # require that AUTOCOMMIT = 1.
+
+    # Check if server1 and server2 have autocommit=0, and if so set the flag
+    #  to 1 and execute the create temporary table query.
+    must_toggle_s1 = not server1.autocommit_set()
+    if must_toggle_s1:
+        server1.toggle_autocommit(enable=True)
     server1.exec_query(tbl1_table)
+
+    must_toggle_s2 = not server2.autocommit_set()
+    if must_toggle_s2:
+        server2.toggle_autocommit(enable=True)
     server2.exec_query(tbl2_table)
 
+    # if the autocommit flag was toggled, return it to its previous value.
+    if must_toggle_s1:
+        server1.toggle_autocommit(enable=False)
+    if must_toggle_s2:
+        server2.toggle_autocommit(enable=False)
+
     return (pri_idx1, pri_idx2, table1_idx, table1_idx_name, diag_msgs)
 
 
@@ -928,7 +981,9 @@ def _make_sum_rows(table, idx_str, span_key_size=8):
 
     # Quote compare table appropriately with backticks
     q_tbl_name = quote_with_backticks(
-        _COMPARE_TABLE_NAME.format(tbl=table.tbl_name))
+        _COMPARE_TABLE_NAME.format(tbl=table.tbl_name),
+        table.sql_mode
+    )
 
     table.server.exec_query(
         _COMPARE_INSERT.format(db=table.q_db_name, compare_tbl=q_tbl_name,
@@ -968,7 +1023,9 @@ def _get_rows_span(table, span, index):
     for row in span:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table.tbl_name),
+            table.sql_mode
+        )
 
         span_rows = server.exec_query(
             _COMPARE_DIFF.format(db=table.q_db_name, compare_tbl=q_tbl_name,
@@ -1016,7 +1073,9 @@ def _get_changed_rows_span(table1, table2, span, index):
     for row in span:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table1.tbl_name),
+            table1.sql_mode
+        )
 
         span_rows = server1.exec_query(
             _COMPARE_DIFF.format(db=table1.q_db_name, compare_tbl=q_tbl_name,
@@ -1032,7 +1091,9 @@ def _get_changed_rows_span(table1, table2, span, index):
     for row in span:
         # Quote compare table appropriately with backticks
         q_tbl_name = quote_with_backticks(
-            _COMPARE_TABLE_NAME.format(tbl=table2.tbl_name))
+            _COMPARE_TABLE_NAME.format(tbl=table2.tbl_name),
+            table2.sql_mode
+        )
 
         span_rows = server2.exec_query(
             _COMPARE_DIFF.format(db=table2.q_db_name, compare_tbl=q_tbl_name,
@@ -1130,7 +1191,10 @@ def _get_formatted_rows(rows, table, fmt='GRID'):
     """
     result_rows = []
     outfile = tempfile.TemporaryFile()
-    print_list(outfile, fmt, table.get_col_names(), rows)
+    to_sql = False
+    if fmt.upper() == 'CSV':
+        to_sql = True
+    print_list(outfile, fmt, table.get_col_names(), rows, to_sql=to_sql)
     outfile.seek(0)
     for line in outfile.readlines():
         result_rows.append(line.strip('\n'))
@@ -1138,6 +1202,95 @@ def _get_formatted_rows(rows, table, fmt='GRID'):
     return result_rows
 
 
+def _generate_data_diff_output(diff_data, table1, table2, used_index, options):
+    """Generates the data difference output.
+
+    This function generates the output data for the found data differences
+    between two tables, according to the provided options (difftype and
+    format).
+
+    diff_data[in]   Tuple with three elements containing the data differences
+                    between two tables. The first element contains the rows on
+                    both tables but with different values, the second contains
+                    the rows found in table1 but not in table2, and the third
+                    contains the rows found in table2 but not in table1.
+    table1[in]      First compared table (source).
+    table2[in]      Second compared table (target).
+    used_index[in]  Index (key) used to identify rows.
+    options[in]     Dictionary of option (format, difftype, compact, etc.).
+
+    Return a list of difference (strings) generated according to the
+    specified options.
+    """
+    difftype = options.get('difftype', 'unified')
+    fmt = options.get('format', 'grid')
+    compact_diff = options.get("compact", False)
+    table1_name = table1.q_table
+    table2_name = table2.q_table
+    changed_rows, extra1, extra2 = diff_data
+    data_diffs = []
+
+    if len(changed_rows) > 0:
+        data_diffs.append("# Data differences found among rows:")
+        # Get original changed/extra rows for each table within the given
+        # span set 'changed_rows' (excluding unchanged rows).
+        # Note: each span can refer to multiple rows.
+        tbl1_rows, tbl2_rows = _get_changed_rows_span(table1, table2,
+                                                      changed_rows,
+                                                      used_index)
+
+        if difftype == 'sql':
+            # Compute SQL diff for changed rows.
+            data_diffs.extend(transform_data(table1, table2, "UPDATE",
+                                             (tbl1_rows[0], tbl2_rows[0])))
+            # Compute SQL diff for extra rows in table 1.
+            if tbl1_rows[1]:
+                data_diffs.extend(transform_data(table1, table2,
+                                                 "DELETE", tbl1_rows[1]))
+            # Compute SQL diff for extra rows in table 2.
+            if tbl2_rows[1]:
+                data_diffs.extend(transform_data(table1, table2,
+                                                 "INSERT", tbl2_rows[1]))
+        else:
+            # Join changed and extra rows for table 1.
+            tbl1_rows = tbl1_rows[0] + tbl1_rows[1]
+            rows1 = _get_formatted_rows(tbl1_rows, table1, fmt)
+            # Join changed and extra rows for table 2.
+            tbl2_rows = tbl2_rows[0] + tbl2_rows[1]
+            rows2 = _get_formatted_rows(tbl2_rows, table2, fmt)
+            # Compute diff for all changes between table 1 and 2.
+            diff_str = _get_diff(rows1, rows2, table1_name, table2_name,
+                                 difftype, compact=compact_diff)
+            if len(diff_str) > 0:
+                data_diffs.extend(diff_str)
+
+    if len(extra1) > 0:
+        # Compute diff for extra rows in table 1.
+        rows = _get_rows_span(table1, extra1, used_index)
+        if difftype == 'sql':
+            data_diffs.extend(transform_data(table1, table2,
+                                             "DELETE", rows))
+        else:
+            data_diffs.append("\n# Rows in {0} not in {1}"
+                              "".format(table1_name, table2_name))
+            res = _get_formatted_rows(rows, table1, fmt)
+            data_diffs.extend(res)
+
+    if len(extra2) > 0:
+        # Compute diff for extra rows in table 2.
+        rows = _get_rows_span(table2, extra2, used_index)
+        if difftype == 'sql':
+            data_diffs.extend(transform_data(table1, table2,
+                                             "INSERT", rows))
+        else:
+            data_diffs.append("\n# Rows in {0} not in {1}"
+                              "".format(table2_name, table1_name))
+            res = _get_formatted_rows(rows, table2, fmt)
+            data_diffs.extend(res)
+
+    return data_diffs
+
+
 def check_consistency(server1, server2, table1_name, table2_name,
                       options=None, diag_msgs=None, reporter=None):
     """Check the data consistency of two tables
@@ -1202,8 +1355,12 @@ def check_consistency(server1, server2, table1_name, table2_name,
        used to form a formatted list of the results for presentation to the
        user.
 
+       Note: The differences output is generated considering the specified
+       changes directions (for server1, server2, or both).
+
     9. The compare databases are destroyed and differences (if any) are
-       returned. A return value of None indicates the data is consistent.
+       returned according to the specified change direction in the options.
+       A return value of (None, None) indicates the data is consistent.
 
     10. Turn binary logging on if turned off in step (1).
 
@@ -1222,16 +1379,16 @@ def check_consistency(server1, server2, table1_name, table2_name,
 
     Returns tuple - string representations of the primary index columns
 
-    Returns None = data is consistent
-            list of differences - data is not consistent
+    Returns a tuple with the list of differences for server1 and/or server2
+            according to the specified direction. If the data is consistent
+            then the tuple (None, None) is returned.
     """
     if options is None:
         options = {}
-    fmt = options.get('format', 'grid')
-    difftype = options.get('difftype', 'unified')
     span_key_size = options.get('span_key_size', DEFAULT_SPAN_KEY_SIZE)
-    compact_diff = options.get("compact", False)
     use_indexes = options.get('use_indexes', None)
+    direction = options.get('changes-for', 'server1')
+    reverse = options.get('reverse', False)
 
     table1 = Table(server1, table1_name)
     table2 = Table(server2, table2_name)
@@ -1252,7 +1409,7 @@ def check_consistency(server1, server2, table1_name, table2_name,
         if checksum1 == checksum2:
             if reporter:
                 reporter.report_state("pass")
-            return None  # No data diffs
+            return None, None  # No data diffs (in any direction)
         else:
             if reporter:
                 reporter.report_state("FAIL")
@@ -1260,18 +1417,32 @@ def check_consistency(server1, server2, table1_name, table2_name,
         if reporter:
             reporter.report_state("SKIP")
 
+    # remove quotations to indexes
+    unq_use_indexes = []
+    if use_indexes:
+        for tbl, index in use_indexes:
+            unq_use_indexes.append((
+                remove_backtick_quoting(tbl, table1.sql_mode),
+                remove_backtick_quoting(index, table1.sql_mode)
+            ))
+            if table1.sql_mode != table2.sql_mode:
+                unq_use_indexes.append((
+                    remove_backtick_quoting(tbl, table2.sql_mode),
+                    remove_backtick_quoting(index, table2.sql_mode)
+                ))
+
     # if given get the unique_key for table_name
     table1_use_indexes = []
     if use_indexes:
         table1_use_indexes.extend(
-            [u_key for tb_name, u_key in use_indexes
-             if table1_name.endswith(".{0}".format(tb_name))]
+            [quote_with_backticks(u_key, table1.sql_mode) for tb_name, u_key
+             in unq_use_indexes if table1.tbl_name == tb_name]
         )
     table2_use_indexes = []
     if use_indexes:
         table2_use_indexes.extend(
-            [u_key for tb_name, u_key in use_indexes
-             if table2_name.endswith(".{0}".format(tb_name))]
+            [quote_with_backticks(u_key, table2.sql_mode) for tb_name, u_key
+             in unq_use_indexes if table2.tbl_name == tb_name]
         )
 
     if options.get('toggle_binlog', 'False'):
@@ -1289,7 +1460,8 @@ def check_consistency(server1, server2, table1_name, table2_name,
         binlog_server1 = False
         binlog_server2 = False
 
-    data_diffs = None
+    data_diffs1 = None
+    data_diffs2 = None
 
     # Now, execute algorithm to find row differences.
     if reporter:
@@ -1300,7 +1472,7 @@ def check_consistency(server1, server2, table1_name, table2_name,
     pri_idx_str1, pri_idx_str2, used_index, used_index_name, msgs = (
         _setup_compare(table1, table2,
                        span_key_size,
-                       use_indexes=(table2_use_indexes, table2_use_indexes))
+                       use_indexes=(table1_use_indexes, table2_use_indexes))
     )
 
     # Add warnings to print them later.
@@ -1320,7 +1492,6 @@ def check_consistency(server1, server2, table1_name, table2_name,
     if len(in1_not2) != 0 or len(in2_not1) != 0:
         table1_diffs = []
         table2_diffs = []
-        data_diffs = []
 
         # Get keys for diffs on table1
         for row in in1_not2:
@@ -1334,61 +1505,17 @@ def check_consistency(server1, server2, table1_name, table2_name,
         changed_rows, extra1, extra2 = get_common_lists(table1_diffs,
                                                         table2_diffs)
 
-        if len(changed_rows) > 0:
-            data_diffs.append("# Data differences found among rows:")
-            # Get original changed/extra rows for each table within the given
-            # span set 'changed_rows' (excluding unchanged rows).
-            # Note: each span can refer to multiple rows.
-            tbl1_rows, tbl2_rows = _get_changed_rows_span(table1, table2,
-                                                          changed_rows,
-                                                          used_index)
-
-            if difftype == 'sql':
-                # Compute SQL diff for changed rows.
-                data_diffs.extend(transform_data(table1, table2, "UPDATE",
-                                                 (tbl1_rows[0], tbl2_rows[0])))
-                # Compute SQL diff for extra rows in table 1.
-                if tbl1_rows[1]:
-                    data_diffs.extend(transform_data(table1, table2,
-                                                     "DELETE", tbl1_rows[1]))
-                # Compute SQL diff for extra rows in table 2.
-                if tbl2_rows[1]:
-                    data_diffs.extend(transform_data(table1, table2,
-                                                     "INSERT", tbl2_rows[1]))
-            else:
-                # Join changed and extra rows for table 1.
-                tbl1_rows = tbl1_rows[0] + tbl1_rows[1]
-                rows1 = _get_formatted_rows(tbl1_rows, table1, fmt)
-                # Join changed and extra rows for table 2.
-                tbl2_rows = tbl2_rows[0] + tbl2_rows[1]
-                rows2 = _get_formatted_rows(tbl2_rows, table2, fmt)
-                # Compute diff for all changes between table 1 and 2.
-                diff_str = _get_diff(rows1, rows2, table1_name, table2_name,
-                                     difftype, compact=compact_diff)
-                if len(diff_str) > 0:
-                    data_diffs.extend(diff_str)
-
-        if len(extra1) > 0:
-            rows = _get_rows_span(table1, extra1, used_index)
-            if difftype == 'sql':
-                data_diffs.extend(transform_data(table1, table2,
-                                                 "DELETE", rows))
-            else:
-                data_diffs.append("\n# Rows in {0} not in {1}"
-                                  "".format(table1_name, table2_name))
-                res = _get_formatted_rows(rows, table1, fmt)
-                data_diffs.extend(res)
-
-        if len(extra2) > 0:
-            rows = _get_rows_span(table2, extra2, used_index)
-            if difftype == 'sql':
-                data_diffs.extend(transform_data(table1, table2,
-                                                 "INSERT", rows))
-            else:
-                data_diffs.append("\n# Rows in {0} not in {1}"
-                                  "".format(table2_name, table1_name))
-                res = _get_formatted_rows(rows, table2, fmt)
-                data_diffs.extend(res)
+        # Generate data differences output according to direction.
+        if direction == 'server1' or reverse:
+            data_diffs1 = _generate_data_diff_output(
+                (changed_rows, extra1, extra2), table1, table2, used_index,
+                options
+            )
+        if direction == 'server2' or reverse:
+            data_diffs2 = _generate_data_diff_output(
+                (changed_rows, extra2, extra1), table2, table1, used_index,
+                options
+            )
 
     if binlog_server1:
         # Commit to avoid error setting sql_log_bin inside a transaction.
@@ -1400,8 +1527,8 @@ def check_consistency(server1, server2, table1_name, table2_name,
         server2.toggle_binlog("ENABLE")
 
     if reporter:
-        if data_diffs:
+        if data_diffs1 or data_diffs2:
             reporter.report_state('FAIL')
         else:
             reporter.report_state('pass')
-    return data_diffs
+    return data_diffs1, data_diffs2
