@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2010, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -283,7 +283,7 @@ def get_local_servers(all_proc=False, start=3306, end=3333,
                 # Look for port on either local or foreign address
                 port = proginfo[1][proginfo[1].find(":") + 1:]
                 if proginfo[1][0] == '0' and port.isdigit():
-                    if int(port) >= start and int(port) <= end:
+                    if int(port) >= int(start) and int(port) <= int(end):
                         processes.append((proginfo[4], port))
                         break
                 if len(proginfo) > 2:
@@ -633,7 +633,6 @@ def stop_running_server(server, wait=10, drop=True):
 
     res = server.show_server_variable("datadir")
     datadir = os.path.normpath(res[0][1])
-
     # Kill all connections so shutdown will work correctly
     res = server.exec_query("SHOW PROCESSLIST")
     for row in res:
@@ -1425,9 +1424,6 @@ class Server(object):
             errors.append("    GTID is not enabled.")
         if not self.check_version_compat(5, 6, 9):
             errors.append("    Server version must be 5.6.9 or greater.")
-        res = self.exec_query("SHOW VARIABLES LIKE 'gtid_executed'")
-        if res == [] or not res[0][0] == "gtid_executed":
-            errors.append("    Missing gtid_executed system variable.")
         if errors:
             errors = "\n".join(errors)
             errors = "\n".join([_GTID_ERROR % (self.host, self.port), errors])
@@ -1666,7 +1662,9 @@ class Server(object):
     def get_all_databases(self, ignore_internal_dbs=True):
         """Return a result set containing all databases on the server
         except for internal databases (mysql, INFORMATION_SCHEMA,
-        PERFORMANCE_SCHEMA)
+        PERFORMANCE_SCHEMA).
+
+        Note: New internal database 'sys' added by default for MySQL 5.7.7+.
 
         Returns result set
         """
@@ -1679,6 +1677,10 @@ class Server(object):
             AND SCHEMA_NAME != 'PERFORMANCE_SCHEMA'
             AND SCHEMA_NAME != 'mysql'
             """
+            # Starting from MySQL 5.7.7, sys schema is installed by default.
+            if self.check_version_compat(5, 7, 7):
+                _GET_DATABASES = "{0} AND SCHEMA_NAME != 'sys'".format(
+                    _GET_DATABASES)
         else:
             _GET_DATABASES = """
             SELECT SCHEMA_NAME
@@ -1791,7 +1793,7 @@ class Server(object):
                 j = create_str.find(" ", i)
                 exist_engine = create_str[i + 7:j]
             else:
-                ## Check if it is a CREATE TABLE LIKE statement
+                # Check if it is a CREATE TABLE LIKE statement
                 is_create_like = (create_str.find("CREATE TABLE {0} LIKE"
                                                   "".format(tbl_name)) == 0)
 
@@ -1977,8 +1979,8 @@ class Server(object):
         Returns bool - True - foreign keys are enabled
         """
         if self.fkeys is None or force:
-            res = self.show_server_variable("foreign_key_checks")
-            self.fkeys = (res is not None) and (res[0][1] == "ON")
+            res = self.exec_query("SELECT @@GLOBAL.foreign_key_checks")
+            self.fkeys = (res is not None) and (res[0][0] == "1")
         return self.fkeys
 
     def disable_foreign_key_checks(self, disable=True):
@@ -2018,7 +2020,7 @@ class Server(object):
                            else if False turn autocommit off (set to 0).
         """
         if enable is None:
-             # Switch autocommit value.
+            # Switch autocommit value.
             if self.autocommit is None:
                 # Get autocommit value if unknown
                 self.autocommit_set()
